#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
生成epub图书
@Project ：spider
@File    ：EpubSpliter.py
@Author  ：Pipiyi
@Date    ：23/5/23 14:43
"""
import os
from ebooklib import epub


class EpubSpliter:
    def __init__(self, title, author, identifier, language="zh-CN"):
        """
        Initialize an EpubSpliter object.
        :param title: Title of the book.
        :param author: Author of the book.
        :param identifier: Identifier of the book.
        :param language: Language of the book (default is 'zh-CN').
        """
        self.book = epub.EpubBook()
        self.book.set_title(title)
        self.book.set_identifier(identifier)
        self.book.add_author(author)
        self.book.set_language(language)
        self.epub_list = []
        self.title = title
        self.is_thumb = False

    def add_cover(self, filename, file_content):
        """
        Add a cover to the book.
        :param filename: The filename of the cover
        :param file_content: The content of the cover (binary format).
        """
        self.book.set_cover(filename, file_content)
        self.is_thumb = True

    def add_item(self, title, file_name, content):
        """
        Add a chapter to the book.
        :param title: The title of the chapter
        :param file_name: The filename of the chapter.
        :param content: The content of the chapter.
        :return:
        """
        chapter = epub.EpubHtml(title=title, file_name=f"{file_name}.xhtml", lang="zh-CN")
        chapter.set_content(content)
        self.book.add_item(chapter)
        self.epub_list.append(chapter)

    def save(self, file_path="", file_name=""):
        """
        Save the book to a file.
        :param file_path: The directory where the book should be saved (default is current directory).
        :param file_name: The name of the file (default is the book title).
        :return:
        """
        # self.book.toc = (epub.Link(chapter.file_name, chapter.title, f"chapter{key}") for key, chapter in
                         # enumerate(self.epub_list))
        self.book.toc = tuple(self.epub_list)
        self.book.spine = ["cover", "nav"] + self.epub_list if self.is_thumb else ["nav"] + self.epub_list

        self.book.add_item(epub.EpubNcx())
        self.book.add_item(epub.EpubNav())

        file_name = file_name if file_name else self.title
        output_path = os.path.join(file_path, f"{file_name}.epub")
        try:
            epub.write_epub(output_path, self.book)
        except Exception as e:
            raise Exception(e)



import random
import requests


class MyRequest:
    USER_AGENTS = [
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/20100101 Firefox/21.0",
        "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/20130331 Firefox/21.0",
        "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/20100101 Firefox/21.0",
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36",
    ]

    def __init__(self, log):
        self.log = log

    @staticmethod
    def random_user_agent():
        return random.choice(MyRequest.USER_AGENTS)

    def header(self, data=None):
        if data is None:
            data = {}
        header = {"user-agent": self.random_user_agent()}
        header.update(data)
        return header

    def request(self, method, url, data=None, params=None, headers=None, timeout=5, verify=False):
        try:
            response = requests.request(
                method,
                url,
                data=data,
                params=params,
                headers=self.header(headers),
                timeout=timeout,
                verify=verify
            )
            if 200 <= response.status_code < 300:
                return response
            else:
                self.log.error(f"url {method} error code:{response.status_code},link:{url}")
                return False
        except requests.RequestException as e:
            self.log.error(f"{method} request error: {e}, URL:{url}")
            return False
        except Exception as e:
            self.log.error(f"{method} request Exception Error: {e}, URL:{url}")
            return False

    def get(self, url, params=None, headers=None, timeout=5, verify=False):
        return self.request("GET", url, params=params, headers=headers, timeout=timeout)

    def post(self, url, data=None, headers=None, timeout=5, verify=False):
        return self.request("POST", url, data=data, headers=headers, timeout=timeout)



import heapq
import os


class Storage:
    def __init__(self):
        """
        :param filename: 文件名称
        :param path: 路径，当为空时为当前目录
        """
        self.filename = ""
        self.path = ""
        self.data = []

    def push(self, item):
        """
        写入
        :param data: dict 结构内必须包含id键用于排序
        """
        heapq.heappush(self.data, (item['id'], item))

    def pop(self):
        """
        以升序获取第一个元素并删除
        :return:
        """
        return heapq.heappop(self.data)

    def has_item(self):
        """
        判断堆是否为空
        :return:bool
        """
        return bool(self.data)

    def save_all(self):
        """
        保存文件
        :return:
        """
        with open(self.filename, "a+") as f:
            while self.data:
                _, data = heapq.heappop(self.data)
                f.write(data["body"])



import logging
import colorlog


class Log:
    # 设置日志格式
    formatter = colorlog.ColoredFormatter(
        "%(log_color)s%(asctime)s - %(levelname)s - %(message)s",
        log_colors={
            "DEBUG": "cyan",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red,bg_white",
        },
    )

    def __init__(self, filename="log"):
        # 它会自动获取当前模块的名称作为 logger 的名称。
        # 这样可以在日志中标识出日志消息来自哪个模块
        self.logger = logging.getLogger(__name__)
        # 设置 logger 对象的日志级别为 DEBUG
        self.logger.setLevel(logging.DEBUG)

        if not self.logger.handlers:
            # 创建日志处理器对象 用于打印台输出
            console_handle = self._create_handler(logging.StreamHandler(), logging.INFO)

            # 创建一个写入文件的处理器对象 用于日志记录
            file_handler = self._create_handler(
                logging.FileHandler(f"{filename}.log"), logging.WARNING
            )

            self.logger.addHandler(console_handle)
            self.logger.addHandler(file_handler)

    @staticmethod
    def _create_handler(handler, level):
        handler.setLevel(level)
        handler.setFormatter(Log.formatter)
        return handler

    def info(self, message, *args):
        self.logger.info(message, *args)

    def debug(self, message, *args):
        self.logger.debug(message, *args)

    def warning(self, message, *args):
        self.logger.warning(message, *args)

    def error(self, message, *args):
        self.logger.error(message, *args)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        for handler in self.logger.handlers:
            handler.close()
            self.logger.removeHandler(handler)



import pymysql
from dbutils.pooled_db import PooledDB
from typing import Dict, List, Union, Tuple, Any


class PooledBase:
    def __init__(self, host, user, password, db, log, port=3306, charset="utf8mb4"):
        self.pool = PooledDB(
            creator=pymysql,  # 使用链接数据库的模块
            maxconnections=0,  # 连接池允许的最大连接数，0和None表示不限制连接数
            mincached=2,  # 初始化时，连接池至少创建的空闲的链接，0表示不创建
            maxcached=5,  # 连接池中最多闲置的链接，0和None不限制
            maxshared=3,
            # 链接池中最多共享的链接数量，0和None表示全部共享。
            # 因为pymysql和MySQLdb等模块的 threadsafety都为1，所以可设为0或None；
            blocking=True,  # 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
            setsession=[],  # 开始会话前执行的命令列表。如：["set datestyle to ...", "set time zone ..."]
            ping=0,
            host=host,
            port=port,
            user=user,
            password=password,
            database=db,
            charset=charset,
        )
        self.log = log

    def execute(self, sql, params=None, is_fetch=True):
        """
        执行sql语句
        :param sql:
        :param params:
        :param is_fetch: bool 是否为查询
        :return:
        """
        with self.pool.connection() as conn:
            with conn.cursor(cursor=pymysql.cursors.DictCursor) as cursor:
                try:
                    cursor.execute(sql, params)
                    if is_fetch:
                        return cursor.fetchall()
                    else:
                        conn.commit()
                        return cursor.lastrowid
                except Exception as e:
                    self.log.error(f"Execute error: {e},sql:{sql}.params:{params}")
                    conn.rollback()

    def select(
            self,
            table: str,
            columns: List[str] = None,
            where: Dict[str, Union[str, int]] = None,
            limit: int = None,
            offset: int = None,
            order_by: str = None,
    ) -> Union[None, Dict, List[Dict]]:
        """
        查询所有数据
        :param table: string 表名
        :param columns: list 值为字符串 字段名 DEMO:
        ["username", "password", "age", "height", "is_active"]
        :param where: dict key：字符串 value: 字符串OR整数 条件 DEMO:
        {"age": 30, "height": 180.5}
        :param limit: 数量
        :param offset: 偏移量
        :param order_by: 排序
        :return: list
        """
        if columns is None:
            columns = ["*"]
        sql = f"SELECT {','.join(columns)} FROM {table}"
        params = None
        if where:
            where_str = " AND ".join([f"{k}=%s" for k in where.keys()])
            sql += f" WHERE {where_str}"
            params = tuple(where.values())

        if order_by:
            sql += f" ORDER BY {order_by}"

        if limit:
            sql += " LIMIT %s"
            params = (limit,) if params is None else params + (limit,)

        if offset:
            sql += " OFFSET %s"
            params = (offset,) if params is None else params + (offset,)

        return self.execute(sql, params)

    def fetch_one(
            self,
            table: str,
            columns: List[str] = None,
            where: Dict[str, Union[str, int]] = None,
    ):
        """
        查询一条
        :param table: string 表名
        :param columns: list 字段名称
        :param where: dict 条件
        :return: dict or None
        """
        result = self.select(table, columns, where, limit=1)
        return result[0] if result else None

    def insert(self, table: str, data: Dict[str, Any]) -> int:
        """
        插入单条
        :param table: string 表名
        :param data: dict 数据 DEMO:
        {"username": "Bob", "password": "password2", "age": 30}
        :return: int 数量
        """
        columns = ", ".join(data.keys())
        placeholders = ",".join(["%s"] * len(data))
        sql = f"INSERT INTO {table} ({columns}) VALUES ({placeholders})"
        params = tuple(data.values())
        return self.execute(sql, params, is_fetch=False)

    def insert_all(self, table: str, data_list: List[Dict[str, Any]]) -> int:
        """
            批量插入
            :param table:string 表名
            :param data_list: 数据 DEMO:
            data_list = [
            {"username": "Alice", "password": "password1", "age": 28},
            {"username": "Bob", "password": "password2", "age": 30},
            {"username": "Charlie", "password": "password3", "age": 25},
        ]
            :return:int 受影响的数目
        """
        if not data_list:
            return 0

        keys = data_list[0].keys()
        placeholders = ",".join(["%s"] * len(keys))
        columns = ",".join(keys)
        sql = f"INSERT INTO {table} ({columns}) VALUES ({placeholders})"

        params_list = [tuple(data.values()) for data in data_list]

        with self.pool.connection() as conn:
            with conn.cursor(cursor=pymysql.cursors.DictCursor) as cursor:
                affected_rows = 0
                try:
                    affected_rows = cursor.executemany(sql, params_list)
                    conn.commit()
                except Exception as e:
                    self.log.error(f"insert all error: {e}")
                    conn.rollback()
                return affected_rows

    def update(self, table: str, data: Dict[str, Any], where: Dict[str, Any]) -> int:
        """
        更新数据
        :param table:string 表名
        :param data: dict 更新的数据
        :param where: dict 条件
        :return: int 受影响的条数
        """
        set_str = " ,".join([f"{key}=%s" for key in data.keys()])
        where_str = " AND ".join([f"{key}=%s" for key in where.keys()])
        sql = f"UPDATE {table} SET {set_str} WHERE {where_str}"
        params = tuple(data.values()) + tuple(where.values())
        return self.execute(sql, params, is_fetch=False)

    def delete(self, table: str, where: Dict[str, Any]) -> int:
        """
        删除
        :param table:string 表名
        :param where: dict 条件
        :return:int 受影响的条数
        """
        where_str = " AND ".join([f"{key}=%s" for key in where.keys()])
        sql = f"DELETE FROM {table} WHERE {where_str}"
        params = tuple(where.values())
        return self.execute(sql, params, is_fetch=False)
